FlagRSPParser = {}; -- string parsing, from TooltipHandler beforehand

-- Function were in the wrong places, just a temporary storage

--[[

FlagRSPParser:ParseString(str, playerID)

- Searches the string for tokens and then replaces them.

]]--
function FlagRSPParser:ParseString(str, playerID) -- TODO responsibility wrong, used in FriendlistFrame.lua and flagrsp.lua
	-- Loops through str for tokens and then replaces them with a corresponding function.
	if ( playerID == "" or playerID == nil ) then
		playerID = "mouseover";
	end
	
	for w in string.gmatch(str, "(%%%a+)") do
		local func = TooltipLookup[w];
		local replace = "%" .. w;
		
		-- If a token has no corresponding function, print an error and returns what we've got.
		if ( func == nil ) then
			Print_Chat("Error: Unknown token: " .. w);
			Print_Chat("In string: " .. str);
			return str;
		end
		
		local result = func(playerID);
		
		if ( result == nil ) then
			result = "";
		end
		str = string.gsub(str, replace, result);
	end
	
	return str;
end


--[[

TooltipHandler:GetGuild(playerID)

- Returns guild of player.

]]--
function TooltipHandler:GetGuild(playerID)
	local guild = "";

	-- capture self-made playerID for Friendlist
	if string.sub(playerID,1,5) == "index" then
		local iname = self:GetCharacterName(playerID, "unit");
		guild = Friendlist:GetGuild(iname);
	else
		local a,b;
		guild, a, b = GetGuildInfo(playerID);
	end
	
	if ( guild == nil ) then 
		guild = ""; 
	end

	return guild;
end


-- --[[

-- TooltipHandler:GetGuild(playerID)

-- - Returns guild of player.

-- ]]--
-- function TooltipHandler:GetGuild(playerID)
	-- local guild = "";

	-- -- capture self-made playerID for Friendlist
	-- if string.sub(playerID,1,5) == "index" then
		-- local iname = self:GetCharacterName(playerID, "unit");
		-- guild = Friendlist:GetGuild(iname);
	-- else
		-- local a,b;
		-- guild, a, b = GetGuildInfo(playerID);
	-- end
	
	-- if ( guild == nil ) then 
		-- guild = ""; 
	-- end

	-- return guild;
-- end


--[[

TooltipHandler:GetPVPRankTooltipText(playerID)

- Returns text for pvp rank text in tooltip.

]]--
function TooltipHandler:GetPVPRankTooltipText(playerID)
	local pvpRankText = "";

	-- capture self-made playerID for Friendlist
	if string.sub(playerID,1,5) == "index" then
		if self:GetGameTitle(playerID) ~= "" then
			pvpRankText = FlagRSP_Locale_PVPRankLine .. self:GetGameTitle(playerID);
		end
	else
		if UnitIsPlayer(playerID) and FlagRSPConfigure:GetPVPRanks() and self:GetGameTitle(playerID) ~= "" then
			pvpRankText = FlagRSP_Locale_PVPRankLine .. self:GetGameTitle(playerID);
		end
	end

	return pvpRankText;
end


--[[

TooltipHandler:GetCharStatusTooltipText(name, playerID)

- Returns text for character status for tooltip line for player <name>.

]]--
function TooltipHandler:GetCharStatusTooltipText(name, playerID, realm) --moved from TooltipHandler
	local charstatus = "";

	if ((playerID == "target" or playerID == "mouseover") and (UnitIsPlayer(playerID) or UnitPlayerControlled(playerID))) or (playerID ~= "target" and playerID ~= "mouseover") then
		local cs = self:GetCharStatusText(name, realm);
		if cs ~= "" then
			charstatus = FlagRSP_Locale_CharStatusLine .. cs;
		end
	end

	return charstatus;
end

--[[

TooltipHandler:GetRPTag(name)

- Returns rp flag for player name (no colours, just the string).

]]--
function TooltipHandler:GetRPTag(name, realm)  -- moved from TooltipHandler
	local rptag = "";

	if TooltipHandler:GetFlagInfo("RPFlag", name, realm) ~= "" and TooltipHandler:GetFlagInfo("RPFlag", name, realm) >= 1 then
		if TooltipHandler:GetFlagInfo("RPFlag", name, realm) == 1 then
			rptag = FlagRSP_Locale_RP;
		elseif TooltipHandler:GetFlagInfo("RPFlag", name, realm) == 2 then
			rptag = FlagRSP_Locale_RP2;
		elseif TooltipHandler:GetFlagInfo("RPFlag", name, realm) == 3 then
			rptag = FlagRSP_Locale_RP3;
		elseif TooltipHandler:GetFlagInfo("RPFlag", name, realm) == 4 then
			rptag = FlagRSP_Locale_RP4;
		end
	end

	return rptag;
end


--[[

- TooltipHandler:GetLevelTooltipText(playerID)

- Returns text for level line in tooltip depending on levelchange.
- Needed: Other level description for friends.

]]--
function TooltipHandler:GetLevelTooltipText(playerID) -- moved from TooltipHandler
	local levelText = "";

	local isplayer;
	local iname = self:GetName(playerID);

	if string.sub(playerID,1,5) == "index" then
		isplayer = true;
	else
		isplayer = UnitIsPlayer(playerID);
	end

	local level = TooltipHandler:GetLevel(playerID);

	if level ~= "" and level ~= 0 then
		if isplayer then
			if ( FlagRSPConfigure:GetLevelDisplay() == "relative" ) then
				levelText = self:GetAlternativeEnemyLevelText(playerID) .. " " .. self:GetRace(playerID) .. " " .. self:GetClass(playerID);
			elseif ( FlagRSPConfigure:GetLevelDisplay() == "color" ) then
				levelText = self:GetAlternateLevelText(playerID) .. " " .. self:GetRace(playerID) .. " " .. self:GetClass(playerID);
			else
				levelText = FlagRSP_Locale_TradLevelLine .. self:GetLevel(playerID) .. " " .. self:GetRace(playerID) .. " " .. self:GetClass(playerID);
			end
		else
			if ( FlagRSPConfigure:GetLevelDisplay() == "relative" ) then
				levelText = FlagRSP_Locale_AltLevelLine .. self:GetAlternativeEnemyLevelText(playerID);
			elseif ( FlagRSPConfigure:GetLevelDisplay() == "color" ) then
				levelText = self:GetAlternateLevelText(playerID);
			else
				levelText = FlagRSP_Locale_TradLevelLine .. self:GetLevel(playerID);
			end

			if ( not UnitIsFriend("player", playerID) and not UnitIsDead(playerID) ) then
				local uct = UnitCreatureType(playerID);
				if uct ~= nil then
					levelText = levelText .. " " .. uct;
				end
			end

			if ( UnitIsDead(playerID) ) then
				levelText = levelText .. FlagRSP_Locale_Corpse;
			end

			if ( string.sub(playerID,1,5) ~= "index" and UnitClassification(playerID) == "elite" ) then
				levelText = levelText .. FlagRSP_Locale_Elite;
			elseif ( string.sub(playerID,1,5) ~= "index" and UnitClassification(playerID) == "worldboss" ) then
				levelText = levelText .. FlagRSP_Locale_Boss;
			elseif ( string.sub(playerID,1,5) ~= "index" and UnitClassification(playerID) == "rare" ) then
				levelText = levelText .. FlagRSP_Locale_Rare;
			elseif ( string.sub(playerID,1,5) ~= "index" and UnitClassification(playerID) == "rareelite" ) then
				levelText = levelText .. FlagRSP_Locale_RareElite;
			end
		end
	end
   
	return levelText;
end


--[[

- TooltipHandler:GetRPTooltipText(name)

- Returns text for rp tag line in tooltip for player name.

]]--
function TooltipHandler:GetRPTooltipText(name, playerID, realm) -- moved from TooltipHandler
	local RPTag = "";

	if playerID == nil then
		playerID = "";
	end

	if ((playerID == "target" or playerID == "mouseover") and (UnitIsPlayer(playerID) or UnitPlayerControlled(playerID))) or (playerID ~= "target" and playerID ~= "mouseover") then
      
		if TooltipHandler:GetFlagInfo("RPFlag", name, realm) ~= "" and TooltipHandler:GetFlagInfo("RPFlag", name, realm) >= 1 then
			RPTag = "|cFF" .. FlagRSPStatic:GetHex(1.0, 1.0, 0.5);

			RPTag = RPTag .. self:GetRPTag(name, realm);

			RPTag = RPTag .. "|r";
		end
	end

	return RPTag;
end


--[[

TooltipHandler:GetCharStatusText(name)

- Returns text for character status for player <name>.

]]--
function TooltipHandler:GetCharStatusText(name, realm) -- moved from TooltipHandler
	local charstatus = "";

    -- local cstatus = self:GetCharStatus(name);
    local cstatus = TooltipHandler:GetFlagInfo("CharFlag", name, realm);
    cstatus = FlagRSPStatic:transformCharFlag(cstatus);
	if cstatus ~= "" then
		charstatus = "|cFF" .. FlagRSPStatic:GetHex(1.0, 1.0, 0.5);
    
		if cstatus == "ooc" then
			charstatus = charstatus .. FlagRSP_Locale_OOC;
		elseif cstatus == "ic" then
			charstatus = charstatus .. FlagRSP_Locale_IC;
		elseif cstatus == "ffa-ic" then
			charstatus = charstatus .. FlagRSP_Locale_FFAIC;
		elseif cstatus == "st" then
			charstatus = charstatus .. FlagRSP_Locale_ST;
		end

		charstatus = charstatus .. "|r";
	end
   
	return charstatus;
end


--[[

TooltipHandler:GetLevelText(playerID)

- Returns text for level line in tooltip depending on levelchange.
- Needed: Other level description for friends.

]]--
function TooltipHandler:GetLevelText(playerID) -- moved from TooltipHandler
	local levelText = "";

	if ( FlagRSPConfigure:GetLevelDisplay() == "relative" ) then 
		levelText = FlagRSP_Locale_AltLevelLine .. self:GetAlternativeEnemyLevelText(playerID);
	else
		if self:GetLevel(playerID) ~= 0 then
			levelText = FRIENDLIST_LOCALE_LevelLine .. self:GetLevel(playerID);
		end
	end
   
	return levelText;
end


--[[

- TooltipHandler:GetCharacterName(playerID, position)

-- Returns the type of name for the character and, if possible, the server.
--  These values may change later.
--  Handled types for <position>: unit, first, last, fullname

]]--
function TooltipHandler:GetCharacterName( playerID, position ) -- TODO moved from TooltipHandler; check (used for friendlist index# name checks, with "unit"
	if ( position == nil or position == "" or playerID == nil or playerID == "" ) then
		return;
	end
	
	local name = "";
	
	if (string.sub(playerID, 1, 5) ~= "index" and position == "unit" and UnitName(playerID) ~= nil) then
		local server;
		name, server = UnitName(playerID);
		return name, server;
	elseif ( string.sub(playerID, 1, 5) == "index" and position == "unit" ) then
		name = TooltipHandler:GetFriendInfo("name", playerID);
		return name;
	elseif ( position == "first" ) then
		local temp = self:GetCharacterName(playerID, "unit")
	
		name = TooltipHandler:GetFlagInfo("Name", temp)
		if ( ( name ~= nil and name ~= " " and name ~= "" ) ) then
			return name;
		elseif ( string.sub(playerID,1,5) == "index" ) then
			name = TooltipHandler:GetFriendInfo("nameT", playerID);
			return name
		else
			if ( UnitName(playerID) ~= nil ) then
				name = UnitName(playerID);
				return name
			end
		end
	elseif ( position == "middle" ) then			
		
	elseif ( position == "last" ) then
		if ( playerID == "mouseover" or playerID == "target" or playerID == "player" ) then
			local temp = self:GetCharacterName(playerID, "unit")

			if ( TooltipHandler:GetFlagInfo("Surname", temp) ~= "" ) then
				name = TooltipHandler:GetFlagInfo("Surname", temp);
				return name;
			elseif ( TooltipHandler:GetFriendInfo("surname", temp) ~= "" and TooltipHandler:GetFriendInfo("surname", temp) ~= nil ) then
				name = TooltipHandler:GetFriendInfo("surname", temp);
				return name;
			end
		end
    elseif ( position == "fullname" ) then
        if ( playerID == "mouseover" or playerID == "target" or playerID == "player" ) then
			local temp = self:GetCharacterName(playerID, "unit")

			if ( TooltipHandler:GetFlagInfo("Fullname", temp) ~= "" ) then
				name = TooltipHandler:GetFlagInfo("Fullname", temp);
				return name;
			elseif ( TooltipHandler:GetFriendInfo("fullname", temp) ~= "" and TooltipHandler:GetFriendInfo("fullname", temp) ~= nil ) then
				name = TooltipHandler:GetFriendInfo("fullname", temp);
				return name;
			end
        end
	end
end